@@ -2,102 +2,27 @@ import logging
 import numpy as np
 import copy
 
-import autoarray as aa
-
 from autoconf import conf
 from autoarray import exc
-from autoarray.dataset import abstract_dataset, preprocess
-from autoarray.structures import arrays
-from autoarray.structures import grids
+from autoarray.structures.arrays.two_d import array_2d
+from autoarray.structures.arrays.two_d import array_2d_util
+from autoarray.structures.grids.two_d import grid_2d
+from autoarray.structures.grids.two_d import grid_2d_irregular
 from autoarray.structures import visibilities as vis
+from autoarray.dataset import abstract_dataset, preprocess
 from autoarray.operators import transformer as trans
 
 
 logger = logging.getLogger(__name__)
 
 
-class AbstractInterferometer(abstract_dataset.AbstractDataset):
-    def __init__(
-        self, visibilities, noise_map, uv_wavelengths, positions=None, name=None
-    ):
-
-        super().__init__(
-            data=visibilities, noise_map=noise_map, positions=positions, name=name
-        )
-
-        self.uv_wavelengths = uv_wavelengths
-
-    @property
-    def visibilities(self):
-        return self.data
-
-    @property
-    def amplitudes(self):
-        return self.visibilities.amplitudes
-
-    @property
-    def phases(self):
-        return self.visibilities.phases
-
-    @property
-    def uv_distances(self):
-        return np.sqrt(
-            np.square(self.uv_wavelengths[:, 0]) + np.square(self.uv_wavelengths[:, 1])
-        )
-
-    def modified_visibilities_from_visibilities(self, visibilities):
-
-        interferometer = copy.deepcopy(self)
-        interferometer.data = vis.Visibilities(visibilities=visibilities)
-        return interferometer
-
-    @property
-    def signal_to_noise_map(self):
-
-        signal_to_noise_map_real = np.divide(
-            np.real(self.data), np.real(self.noise_map)
-        )
-        signal_to_noise_map_real[signal_to_noise_map_real < 0] = 0.0
-        signal_to_noise_map_imag = np.divide(
-            np.imag(self.data), np.imag(self.noise_map)
-        )
-        signal_to_noise_map_imag[signal_to_noise_map_imag < 0] = 0.0
-
-        return signal_to_noise_map_real + 1j * signal_to_noise_map_imag
-
-    def signal_to_noise_limited_from(self, signal_to_noise_limit):
-
-        interferometer = copy.deepcopy(self)
-
-        noise_map_limit_real = np.where(
-            np.real(self.signal_to_noise_map) > signal_to_noise_limit,
-            np.real(self.visibilities) / signal_to_noise_limit,
-            np.real(self.noise_map),
-        )
-
-        noise_map_limit_imag = np.where(
-            np.imag(self.signal_to_noise_map) > signal_to_noise_limit,
-            np.imag(self.visibilities) / signal_to_noise_limit,
-            np.imag(self.noise_map),
-        )
-
-        interferometer.noise_map = vis.VisibilitiesNoiseMap(
-            visibilities=noise_map_limit_real + 1j * noise_map_limit_imag
-        )
-
-        return interferometer
-
-
-class AbstractSettingsMaskedInterferometer(
-    abstract_dataset.AbstractSettingsMaskedDataset
-):
-
+class SettingsInterferometer(abstract_dataset.AbstractSettingsDataset):
     def __init__(
         self,
-        grid_class=grids.Grid2D,
-        grid_inversion_class=grids.Grid2D,
-        sub_size=2,
-        sub_size_inversion=2,
+        grid_class=grid_2d.Grid2D,
+        grid_inversion_class=grid_2d.Grid2D,
+        sub_size=1,
+        sub_size_inversion=4,
         fractional_accuracy=0.9999,
         sub_steps=None,
         pixel_scales_interp=None,
@@ -149,116 +74,41 @@ class AbstractSettingsMaskedInterferometer(
 
         self.transformer_class = transformer_class
 
-    @property
-    def tag_no_inversion(self):
-        return (
-            f"{conf.instance['notation']['settings_tags']['interferometer']['interferometer']}["
-            f"{self.grid_tag_no_inversion}"
-            f"{self.transformer_tag}"
-            f"{self.signal_to_noise_limit_tag}]"
-        )
-
-    @property
-    def tag_with_inversion(self):
-        return (
-            f"{conf.instance['notation']['settings_tags']['interferometer']['interferometer']}["
-            f"{self.grid_tag_with_inversion}"
-            f"{self.transformer_tag}"
-            f"{self.signal_to_noise_limit_tag}]"
-        )
-
-    @property
-    def transformer_tag(self):
-        """Generate an image psf shape tag, to customize phase names based on size of the image PSF that the original PSF \
-        is trimmed to for faster run times.
-
-        This changes the phase settings folder as follows:
-
-        image_psf_shape = 1 -> settings
-        image_psf_shape = 2 -> settings_image_psf_shape_2
-        image_psf_shape = 2 -> settings_image_psf_shape_2
-        """
-        if self.transformer_class is None:
-            return ""
-        return f"__{conf.instance['notation']['settings_tags']['interferometer'][self.transformer_class.__name__]}"
-
-
-class AbstractMaskedInterferometer(abstract_dataset.AbstractMaskedDataset):
 
+class Interferometer(abstract_dataset.AbstractDataset):
     def __init__(
         self,
-        interferometer,
-        visibilities_mask,
+        visibilities,
+        noise_map,
+        uv_wavelengths,
         real_space_mask,
-        settings=AbstractSettingsMaskedInterferometer(),
+        settings=SettingsInterferometer(),
+        name=None,
     ):
-        """
-        The lens dataset is the collection of data_type (image, noise-map), a mask, grid, convolver \
-        and other utilities that are used for modeling and fitting an image of a strong lens.
 
-        Whilst the image, noise-map, etc. are loaded in 2D, the lens dataset creates reduced 1D arrays of each \
-        for lens calculations.
-
-        Parameters
-        ----------
-        imaging: im.Imaging
-            The imaging data_type all in 2D (the image, noise-map, etc.)
-        real_space_mask: msk.Mask2D
-            The 2D mask that is applied to the image.
-        """
+        self.real_space_mask = real_space_mask
 
         super().__init__(
-            dataset=interferometer, mask=real_space_mask, settings=settings
+            data=visibilities, noise_map=noise_map, name=name, settings=settings
         )
 
+        self.uv_wavelengths = uv_wavelengths
+
         self.transformer = self.settings.transformer_class(
-            uv_wavelengths=interferometer.uv_wavelengths,
-            real_space_mask=real_space_mask,
+            uv_wavelengths=uv_wavelengths, real_space_mask=real_space_mask
         )
 
-        self.visibilities = interferometer.visibilities
-        self.noise_map = interferometer.noise_map
-        self.visibilities_mask = visibilities_mask
-
-    @property
-    def interferometer(self):
-        return self.dataset
-
-    @property
-    def data(self):
-        return self.visibilities
-
-    @property
-    def uv_distances(self):
-        return self.interferometer.uv_distances
-
-    @property
-    def real_space_mask(self):
-        return self.mask
-
-    def signal_to_noise_map(self):
-        return self.visibilities / self.noise_map
-
-    def modify_noise_map(self, noise_map):
-
-        masked_interferometer = copy.deepcopy(self)
-
-        masked_interferometer.noise_map = noise_map
-
-        return masked_interferometer
-
-
-class Interferometer(AbstractInterferometer):
     @classmethod
     def from_fits(
         cls,
         visibilities_path,
         noise_map_path,
         uv_wavelengths_path,
+        real_space_mask,
         visibilities_hdu=0,
         noise_map_hdu=0,
         uv_wavelengths_hdu=0,
-        positions_path=None,
+        settings=SettingsInterferometer(),
     ):
         """Factory for loading the interferometer data_type from .fits files, as well as computing properties like the noise-map,
         exposure-time map, etc. from the interferometer-data_type.
@@ -270,33 +120,129 @@ class Interferometer(AbstractInterferometer):
         ----------
         """
 
-        visibilities = aa.Visibilities.from_fits(
+        visibilities = vis.Visibilities.from_fits(
             file_path=visibilities_path, hdu=visibilities_hdu
         )
 
-        noise_map = aa.VisibilitiesNoiseMap.from_fits(
+        noise_map = vis.VisibilitiesNoiseMap.from_fits(
             file_path=noise_map_path, hdu=noise_map_hdu
         )
 
-        uv_wavelengths = aa.util.array.numpy_array_2d_from_fits(
+        uv_wavelengths = array_2d_util.numpy_array_2d_from_fits(
             file_path=uv_wavelengths_path, hdu=uv_wavelengths_hdu
         )
 
-        if positions_path is not None:
-
-            positions = grids.Grid2DIrregular.from_json(file_path=positions_path)
-
-        else:
-
-            positions = None
-
         return Interferometer(
+            real_space_mask=real_space_mask,
             visibilities=visibilities,
             noise_map=noise_map,
             uv_wavelengths=uv_wavelengths,
-            positions=positions,
+            settings=settings,
         )
 
+    def apply_settings(self, settings):
+
+        return Interferometer(
+            visibilities=self.visibilities,
+            noise_map=self.noise_map,
+            uv_wavelengths=self.uv_wavelengths,
+            real_space_mask=self.real_space_mask,
+            settings=settings,
+            name=self.name,
+        )
+
+    @property
+    def mask(self):
+        return self.real_space_mask
+
+    @property
+    def visibilities(self):
+        return self.data
+
+    @property
+    def amplitudes(self):
+        return self.visibilities.amplitudes
+
+    @property
+    def phases(self):
+        return self.visibilities.phases
+
+    @property
+    def uv_distances(self):
+        return np.sqrt(
+            np.square(self.uv_wavelengths[:, 0]) + np.square(self.uv_wavelengths[:, 1])
+        )
+
+    @property
+    def dirty_image(self):
+        return self.transformer.image_from_visibilities(visibilities=self.visibilities)
+
+    @property
+    def dirty_noise_map(self):
+        return self.transformer.image_from_visibilities(visibilities=self.noise_map)
+
+    @property
+    def dirty_signal_to_noise_map(self):
+        return self.transformer.image_from_visibilities(
+            visibilities=self.signal_to_noise_map
+        )
+
+    @property
+    def dirty_inverse_noise_map(self):
+        return self.transformer.image_from_visibilities(
+            visibilities=self.inverse_noise_map
+        )
+
+    def modified_visibilities_from_visibilities(self, visibilities):
+
+        interferometer = copy.deepcopy(self)
+        interferometer.data = vis.Visibilities(visibilities=visibilities)
+        return interferometer
+
+    @property
+    def signal_to_noise_map(self):
+
+        signal_to_noise_map_real = np.divide(
+            np.real(self.data), np.real(self.noise_map)
+        )
+        signal_to_noise_map_real[signal_to_noise_map_real < 0] = 0.0
+        signal_to_noise_map_imag = np.divide(
+            np.imag(self.data), np.imag(self.noise_map)
+        )
+        signal_to_noise_map_imag[signal_to_noise_map_imag < 0] = 0.0
+
+        return signal_to_noise_map_real + 1j * signal_to_noise_map_imag
+
+    def signal_to_noise_limited_from(self, signal_to_noise_limit, mask=None):
+
+        interferometer = copy.deepcopy(self)
+
+        noise_map_limit_real = np.where(
+            np.real(self.signal_to_noise_map) > signal_to_noise_limit,
+            np.real(self.visibilities) / signal_to_noise_limit,
+            np.real(self.noise_map),
+        )
+
+        noise_map_limit_imag = np.where(
+            np.imag(self.signal_to_noise_map) > signal_to_noise_limit,
+            np.imag(self.visibilities) / signal_to_noise_limit,
+            np.imag(self.noise_map),
+        )
+
+        interferometer.noise_map = vis.VisibilitiesNoiseMap(
+            visibilities=noise_map_limit_real + 1j * noise_map_limit_imag
+        )
+
+        return interferometer
+
+    def modify_noise_map(self, noise_map):
+
+        interferometer = copy.deepcopy(self)
+
+        interferometer.noise_map = noise_map
+
+        return interferometer
+
     def output_to_fits(
         self,
         visibilities_path=None,
@@ -314,23 +260,13 @@ class Interferometer(AbstractInterferometer):
             self.noise_map.output_to_fits(file_path=noise_map_path, overwrite=overwrite)
 
         if self.uv_wavelengths is not None and uv_wavelengths_path is not None:
-            aa.util.array.numpy_array_2d_to_fits(
+            array_2d_util.numpy_array_2d_to_fits(
                 array_2d=self.uv_wavelengths,
                 file_path=uv_wavelengths_path,
                 overwrite=overwrite,
             )
 
 
-class SettingsMaskedInterferometer(AbstractSettingsMaskedInterferometer):
-
-    pass
-
-
-class MaskedInterferometer(AbstractMaskedInterferometer):
-
-    pass
-
-
 class AbstractSimulatorInterferometer:
     def __init__(
         self,
@@ -348,7 +284,7 @@ class AbstractSimulatorInterferometer:
         Parameters
         ----------
         real_space_shape_native : (int, int)
-            The shape of the observation. Note that we do not simulator a full Imaging frame (e.g. 2000 x 2000 pixels for \
+            The shape of the observation. Note that we do not simulator a full Imaging array (e.g. 2000 x 2000 pixels for \
             Hubble imaging), but instead just a cut-out around the strong lens.
         real_space_pixel_scales : float
             The size of each pixel in scaled units.
@@ -396,7 +332,7 @@ class AbstractSimulatorInterferometer:
             uv_wavelengths=self.uv_wavelengths, real_space_mask=image.mask
         )
 
-        background_sky_map = arrays.Array2D.full(
+        background_sky_map = array_2d.Array2D.full(
             fill_value=self.background_sky_level,
             shape_native=image.shape_native,
             pixel_scales=image.pixel_scales,
@@ -429,6 +365,7 @@ class AbstractSimulatorInterferometer:
             visibilities=visibilities,
             noise_map=noise_map,
             uv_wavelengths=transformer.uv_wavelengths,
+            real_space_mask=image.mask,
             name=name,
         )
 
