@@ -1,9 +1,8 @@
 from autoarray.structures.arrays import abstract_array
-from autoarray.structures.frames import abstract_frame
-from autoarray.structures import arrays
-from autoarray.structures import frames as f
-from autoarray.structures import region as reg
-from autoarray.util import array_util
+from autoarray.structures.arrays.two_d import array_2d_util
+from autoarray.structures.arrays.two_d import array_2d
+from autoarray.layout import layout as lo, layout_util
+from autoarray.layout import region as reg
 from autoarray import exc
 
 from astropy.io import fits
@@ -38,13 +37,9 @@ def array_eps_to_counts(array_eps, bscale, bzero):
     return (array_eps - bzero) / bscale
 
 
-class ArrayACS(arrays.Array2D):
-
-    pass
-
-
-class FrameACS(f.Frame2D, ArrayACS):
-    """An ACS frame consists of four quadrants ('A', 'B', 'C', 'D') which have the following layout:
+class Array2DACS(array_2d.Array2D):
+    """
+    An ACS array consists of four quadrants ('A', 'B', 'C', 'D') which have the following layout:
 
        <--------S-----------   ---------S----------->
     [] [========= 2 =========] [========= 3 =========] []          /\
@@ -74,7 +69,7 @@ class FrameACS(f.Frame2D, ArrayACS):
 
         hdu = fits_hdu_from_quadrant_letter(quadrant_letter=quadrant_letter)
 
-        array = array_util.numpy_array_2d_from_fits(file_path=file_path, hdu=hdu)
+        array = array_2d_util.numpy_array_2d_from_fits(file_path=file_path, hdu=hdu)
 
         return cls.from_ccd(array_electrons=array, quadrant_letter=quadrant_letter)
 
@@ -85,8 +80,6 @@ class FrameACS(f.Frame2D, ArrayACS):
         quadrant_letter,
         parallel_size=2068,
         serial_size=2072,
-        serial_prescan_size=24,
-        parallel_overscan_size=20,
         exposure_info=None,
     ):
         """
@@ -100,19 +93,13 @@ class FrameACS(f.Frame2D, ArrayACS):
 
             return cls.left(
                 array_electrons=array_electrons[0:parallel_size, 0:serial_size],
-                parallel_size=parallel_size,
-                serial_size=serial_size,
-                serial_prescan_size=serial_prescan_size,
-                parallel_overscan_size=parallel_overscan_size,
                 exposure_info=exposure_info,
             )
         elif quadrant_letter == "A" or quadrant_letter == "D":
             return cls.right(
-                array=array_electrons[0:parallel_size, serial_size : serial_size * 2],
-                parallel_size=parallel_size,
-                serial_size=serial_size,
-                serial_prescan_size=serial_prescan_size,
-                parallel_overscan_size=parallel_overscan_size,
+                array_electrons=array_electrons[
+                    0:parallel_size, serial_size : serial_size * 2
+                ],
                 exposure_info=exposure_info,
             )
         else:
@@ -121,15 +108,7 @@ class FrameACS(f.Frame2D, ArrayACS):
             )
 
     @classmethod
-    def left(
-        cls,
-        array_electrons,
-        parallel_size=2068,
-        serial_size=2072,
-        serial_prescan_size=24,
-        parallel_overscan_size=20,
-        exposure_info=None,
-    ):
+    def left(cls, array_electrons, exposure_info=None):
         """
         Use an input array of the left quadrant in electrons and perform the rotations required to give correct
         arctic clocking.
@@ -137,37 +116,17 @@ class FrameACS(f.Frame2D, ArrayACS):
         See the docstring of the `FrameACS` class for a complete description of the Euclid FPA, quadrants and
         rotations.
         """
-        parallel_overscan = reg.Region2D(
-            (
-                parallel_size - parallel_overscan_size,
-                parallel_size,
-                serial_prescan_size,
-                serial_size,
-            )
+
+        array_electrons = layout_util.rotate_array_from_roe_corner(
+            array=array_electrons, roe_corner=(1, 0)
         )
 
-        serial_prescan = reg.Region2D((0, parallel_size, 0, serial_prescan_size))
-
         return cls.manual(
-            array=array_electrons,
-            roe_corner=(1, 0),
-            scans=abstract_frame.Scans(
-                parallel_overscan=parallel_overscan, serial_prescan=serial_prescan
-            ),
-            exposure_info=exposure_info,
-            pixel_scales=0.05,
+            array=array_electrons, exposure_info=exposure_info, pixel_scales=0.05
         )
 
     @classmethod
-    def right(
-        cls,
-        array,
-        parallel_size=2068,
-        serial_size=2072,
-        parallel_overscan_size=20,
-        serial_prescan_size=51,
-        exposure_info=None,
-    ):
+    def right(cls, array_electrons, exposure_info=None):
         """
         Use an input array of the right quadrant in electrons and perform the rotations required to give correct
         arctic clocking.
@@ -175,31 +134,18 @@ class FrameACS(f.Frame2D, ArrayACS):
         See the docstring of the `FrameACS` class for a complete description of the Euclid FPA, quadrants and
         rotations.
         """
-        parallel_overscan = reg.Region2D(
-            (
-                parallel_size - parallel_overscan_size,
-                parallel_size,
-                0,
-                serial_size - serial_prescan_size,
-            )
-        )
 
-        serial_prescan = reg.Region2D(
-            (0, parallel_size, serial_size - serial_prescan_size, serial_size)
+        array_electrons = layout_util.rotate_array_from_roe_corner(
+            array=array_electrons, roe_corner=(1, 1)
         )
 
         return cls.manual(
-            array=array,
-            roe_corner=(1, 1),
-            scans=abstract_frame.Scans(
-                parallel_overscan=parallel_overscan, serial_prescan=serial_prescan
-            ),
-            exposure_info=exposure_info,
-            pixel_scales=0.05,
+            array=array_electrons, exposure_info=exposure_info, pixel_scales=0.05
         )
 
     def update_fits(self, original_file_path, new_file_path):
-        """Output the array to a .fits file.
+        """
+        Output the array to a .fits file.
 
         Parameters
         ----------
@@ -221,7 +167,9 @@ class FrameACS(f.Frame2D, ArrayACS):
 
         hdulist = fits.open(new_file_path)
 
-        hdulist[self.exposure_info.hdu].data = self.original_orientation
+        hdulist[self.exposure_info.hdu].data = self.layout.original_orientation_from(
+            array=self
+        )
 
         ext_header = hdulist[4].header
         bscale = ext_header["BSCALE"]
@@ -231,8 +179,9 @@ class FrameACS(f.Frame2D, ArrayACS):
         hdulist.writeto(new_file_path)
 
 
-class ImageACS(FrameACS, ArrayACS):
-    """The layout of an ACS frame and image is given in `FrameACS`.
+class ImageACS(Array2DACS):
+    """
+    The layout of an ACS array and image is given in `FrameACS`.
 
     This class handles specifically the image of an ACS observation, assuming that it contains specific
     header info.
@@ -293,7 +242,7 @@ class ImageACS(FrameACS, ArrayACS):
     @staticmethod
     def array_converted_to_electrons_from_fits(file_path, hdu, exposure_info):
 
-        array = array_util.numpy_array_2d_from_fits(
+        array = array_2d_util.numpy_array_2d_from_fits(
             file_path=file_path, hdu=hdu, do_not_scale_image_data=True
         )
 
@@ -305,6 +254,112 @@ class ImageACS(FrameACS, ArrayACS):
             ) + exposure_info.bzero
 
 
+class Layout2DACS(lo.Layout2D):
+    @classmethod
+    def from_ccd(
+        cls,
+        quadrant_letter,
+        parallel_size=2068,
+        serial_size=2072,
+        serial_prescan_size=24,
+        parallel_overscan_size=20,
+    ):
+        """
+        Using an input array of both quadrants in electrons, use the quadrant letter to extract the quadrant from the
+        full CCD and perform the rotations required to give correct arctic.
+
+        See the docstring of the `FrameACS` class for a complete description of the Euclid FPA, quadrants and
+        rotations.
+        """
+        if quadrant_letter == "B" or quadrant_letter == "C":
+
+            return cls.left(
+                parallel_size=parallel_size,
+                serial_size=serial_size,
+                serial_prescan_size=serial_prescan_size,
+                parallel_overscan_size=parallel_overscan_size,
+            )
+        elif quadrant_letter == "A" or quadrant_letter == "D":
+            return cls.right(
+                parallel_size=parallel_size,
+                serial_size=serial_size,
+                serial_prescan_size=serial_prescan_size,
+                parallel_overscan_size=parallel_overscan_size,
+            )
+        else:
+            raise exc.FrameException(
+                "Quadrant letter for FrameACS must be A, B, C or D."
+            )
+
+    @classmethod
+    def left(
+        cls,
+        parallel_size=2068,
+        serial_size=2072,
+        serial_prescan_size=24,
+        parallel_overscan_size=20,
+    ):
+        """
+        Use an input array of the left quadrant in electrons and perform the rotations required to give correct
+        arctic clocking.
+
+        See the docstring of the `FrameACS` class for a complete description of the Euclid FPA, quadrants and
+        rotations.
+        """
+        parallel_overscan = reg.Region2D(
+            (
+                parallel_size - parallel_overscan_size,
+                parallel_size,
+                serial_prescan_size,
+                serial_size,
+            )
+        )
+
+        serial_prescan = reg.Region2D((0, parallel_size, 0, serial_prescan_size))
+
+        return lo.Layout2D.rotated_from_roe_corner(
+            roe_corner=(1, 0),
+            shape_native=(parallel_size, serial_size),
+            parallel_overscan=parallel_overscan,
+            serial_prescan=serial_prescan,
+        )
+
+    @classmethod
+    def right(
+        cls,
+        parallel_size=2068,
+        serial_size=2072,
+        parallel_overscan_size=20,
+        serial_prescan_size=51,
+    ):
+        """
+        Use an input array of the right quadrant in electrons and perform the rotations required to give correct
+        arctic clocking.
+
+        See the docstring of the `FrameACS` class for a complete description of the Euclid FPA, quadrants and
+        rotations.
+        """
+        parallel_overscan = reg.Region2D(
+            (
+                parallel_size - parallel_overscan_size,
+                parallel_size,
+                0,
+                serial_size - serial_prescan_size,
+            )
+        )
+
+        serial_prescan = reg.Region2D(
+            (0, parallel_size, serial_size - serial_prescan_size, serial_size)
+        )
+
+        return lo.Layout2D.rotated_from_roe_corner(
+            roe_corner=(1, 1),
+            shape_native=(parallel_size, serial_size),
+            parallel_overscan=parallel_overscan,
+            serial_prescan=serial_prescan,
+        )
+
+
 class ExposureInfoACS(abstract_array.ExposureInfo):
     def __init__(
         self,
@@ -317,7 +372,7 @@ class ExposureInfoACS(abstract_array.ExposureInfo):
         hdu=None,
     ):
 
-        super(ExposureInfoACS, self).__init__(
+        super().__init__(
             exposure_time=exposure_time,
             date_of_observation=date_of_observation,
             time_of_observation=time_of_observation,
