@@ -1,6 +1,6 @@
-from autoarray.structures.frames import abstract_frame
-from autoarray.structures import frames as f
-from autoarray.structures import region as reg
+from autoarray.layout import layout as lo, layout_util
+from autoarray.layout import region as reg
+from autoarray.structures.arrays.two_d import array_2d
 
 
 def roe_corner_from(ccd_id, quadrant_id):
@@ -25,7 +25,7 @@ def roe_corner_from(ccd_id, quadrant_id):
         return (1, 1)
 
 
-class FrameEuclid(f.Frame2D):
+class Array2DEuclid(array_2d.Array2D):
     """
     In the Euclid FPA, the quadrant id ('E', 'F', 'G', 'H') depends on whether the CCD is located
     on the left side (rows 1-3) or right side (rows 4-6) of the FPA:
@@ -82,7 +82,7 @@ class FrameEuclid(f.Frame2D):
         Use an input array of a Euclid quadrant and its corresponding .fits file header to rotate the quadrant to
         the correct orientation for arCTIc clocking.
 
-        See the docstring of the `FrameEuclid` class for a complete description of the Euclid FPA, quadrants and
+        See the docstring of the `Array2DEuclid` class for a complete description of the Euclid FPA, quadrants and
         rotations.
         """
 
@@ -108,10 +108,104 @@ class FrameEuclid(f.Frame2D):
             parallel_overscan_size=parallel_overscan_size,
         )
 
+    @classmethod
+    def from_ccd_and_quadrant_id(cls, array, ccd_id, quadrant_id):
+        """
+        Use an input array of a Euclid quadrant, its ccd_id and quadrant_id  to rotate the quadrant to
+        the correct orientation for arCTIc clocking.
+
+        See the docstring of the `Array2DEuclid` class for a complete description of the Euclid FPA, quadrants and
+        rotations.
+        """
+
+        row_index = ccd_id[-1]
+
+        if (row_index in "123") and (quadrant_id == "E"):
+            return Array2DEuclid.bottom_left(array_electrons=array)
+        elif (row_index in "123") and (quadrant_id == "F"):
+            return Array2DEuclid.bottom_right(array_electrons=array)
+        elif (row_index in "123") and (quadrant_id == "G"):
+            return Array2DEuclid.top_right(array_electrons=array)
+        elif (row_index in "123") and (quadrant_id == "H"):
+            return Array2DEuclid.top_left(array_electrons=array)
+        elif (row_index in "456") and (quadrant_id == "E"):
+            return Array2DEuclid.top_right(array_electrons=array)
+        elif (row_index in "456") and (quadrant_id == "F"):
+            return Array2DEuclid.top_left(array_electrons=array)
+        elif (row_index in "456") and (quadrant_id == "G"):
+            return Array2DEuclid.bottom_left(array_electrons=array)
+        elif (row_index in "456") and (quadrant_id == "H"):
+            return Array2DEuclid.bottom_right(array_electrons=array)
+
+    @classmethod
+    def top_left(cls, array_electrons):
+        """
+        Use an input array of a Euclid quadrant corresponding to the top-left of a Euclid CCD and rotate the quadrant
+        to the correct orientation for arCTIc clocking.
+
+        See the docstring of the `Array2DEuclid` class for a complete description of the Euclid FPA, quadrants and
+        rotations.
+        """
+
+        array_electrons = layout_util.rotate_array_from_roe_corner(
+            array=array_electrons, roe_corner=(0, 0)
+        )
+
+        return cls.manual(array=array_electrons, pixel_scales=0.1)
+
+    @classmethod
+    def top_right(cls, array_electrons):
+        """
+        Use an input array of a Euclid quadrant corresponding the top-left of a Euclid CCD and rotate the  quadrant to
+        the correct orientation for arCTIc clocking.
+
+        See the docstring of the `Array2DEuclid` class for a complete description of the Euclid FPA, quadrants and
+        rotations.
+        """
+
+        array_electrons = layout_util.rotate_array_from_roe_corner(
+            array=array_electrons, roe_corner=(0, 1)
+        )
+
+        return cls.manual(array=array_electrons, pixel_scales=0.1)
+
+    @classmethod
+    def bottom_left(cls, array_electrons):
+        """
+        Use an input array of a Euclid quadrant corresponding to the bottom-left of a Euclid CCD and rotate the
+        quadrant to the correct orientation for arCTIc clocking.
+
+        See the docstring of the `Array2DEuclid` class for a complete description of the Euclid FPA, quadrants and
+        rotations.
+        """
+
+        array_electrons = layout_util.rotate_array_from_roe_corner(
+            array=array_electrons, roe_corner=(1, 0)
+        )
+
+        return cls.manual(array=array_electrons, pixel_scales=0.1)
+
+    @classmethod
+    def bottom_right(cls, array_electrons):
+        """
+        Use an input array of a Euclid quadrant corresponding to the bottom-right of a Euclid CCD and rotate the
+        quadrant to the correct orientation for arCTIc clocking.
+
+        See the docstring of the `Array2DEuclid` class for a complete description of the Euclid FPA, quadrants and
+        rotations.
+        """
+
+        array_electrons = layout_util.rotate_array_from_roe_corner(
+            array=array_electrons, roe_corner=(1, 1)
+        )
+
+        return cls.manual(array=array_electrons, pixel_scales=0.1)
+
+
+class Layout2DEuclid(lo.Layout2D):
     @classmethod
     def from_ccd_and_quadrant_id(
         cls,
-        array,
         ccd_id,
         quadrant_id,
         parallel_size=2086,
@@ -124,15 +218,14 @@ class FrameEuclid(f.Frame2D):
         Use an input array of a Euclid quadrant, its ccd_id and quadrant_id  to rotate the quadrant to
         the correct orientation for arCTIc clocking.
 
-        See the docstring of the `FrameEuclid` class for a complete description of the Euclid FPA, quadrants and
+        See the docstring of the `Array2DEuclid` class for a complete description of the Euclid FPA, quadrants and
         rotations.
         """
 
         row_index = ccd_id[-1]
 
         if (row_index in "123") and (quadrant_id == "E"):
-            return FrameEuclid.bottom_left(
-                array=array,
+            return Layout2DEuclid.bottom_left(
                 parallel_size=parallel_size,
                 serial_size=serial_size,
                 serial_prescan_size=serial_prescan_size,
@@ -140,8 +233,7 @@ class FrameEuclid(f.Frame2D):
                 parallel_overscan_size=parallel_overscan_size,
             )
         elif (row_index in "123") and (quadrant_id == "F"):
-            return FrameEuclid.bottom_right(
-                array=array,
+            return Layout2DEuclid.bottom_right(
                 parallel_size=parallel_size,
                 serial_size=serial_size,
                 serial_prescan_size=serial_prescan_size,
@@ -149,8 +241,7 @@ class FrameEuclid(f.Frame2D):
                 parallel_overscan_size=parallel_overscan_size,
             )
         elif (row_index in "123") and (quadrant_id == "G"):
-            return FrameEuclid.top_right(
-                array=array,
+            return Layout2DEuclid.top_right(
                 parallel_size=parallel_size,
                 serial_size=serial_size,
                 serial_prescan_size=serial_prescan_size,
@@ -158,8 +249,7 @@ class FrameEuclid(f.Frame2D):
                 parallel_overscan_size=parallel_overscan_size,
             )
         elif (row_index in "123") and (quadrant_id == "H"):
-            return FrameEuclid.top_left(
-                array_electrons=array,
+            return Layout2DEuclid.top_left(
                 parallel_size=parallel_size,
                 serial_size=serial_size,
                 serial_prescan_size=serial_prescan_size,
@@ -167,8 +257,7 @@ class FrameEuclid(f.Frame2D):
                 parallel_overscan_size=parallel_overscan_size,
             )
         elif (row_index in "456") and (quadrant_id == "E"):
-            return FrameEuclid.top_right(
-                array=array,
+            return Layout2DEuclid.top_right(
                 parallel_size=parallel_size,
                 serial_size=serial_size,
                 serial_prescan_size=serial_prescan_size,
@@ -176,8 +265,7 @@ class FrameEuclid(f.Frame2D):
                 parallel_overscan_size=parallel_overscan_size,
             )
         elif (row_index in "456") and (quadrant_id == "F"):
-            return FrameEuclid.top_left(
-                array_electrons=array,
+            return Layout2DEuclid.top_left(
                 parallel_size=parallel_size,
                 serial_size=serial_size,
                 serial_prescan_size=serial_prescan_size,
@@ -185,8 +273,7 @@ class FrameEuclid(f.Frame2D):
                 parallel_overscan_size=parallel_overscan_size,
             )
         elif (row_index in "456") and (quadrant_id == "G"):
-            return FrameEuclid.bottom_left(
-                array=array,
+            return Layout2DEuclid.bottom_left(
                 parallel_size=parallel_size,
                 serial_size=serial_size,
                 serial_prescan_size=serial_prescan_size,
@@ -194,8 +281,7 @@ class FrameEuclid(f.Frame2D):
                 parallel_overscan_size=parallel_overscan_size,
             )
         elif (row_index in "456") and (quadrant_id == "H"):
-            return FrameEuclid.bottom_right(
-                array=array,
+            return Layout2DEuclid.bottom_right(
                 parallel_size=parallel_size,
                 serial_size=serial_size,
                 serial_prescan_size=serial_prescan_size,
@@ -203,122 +289,6 @@ class FrameEuclid(f.Frame2D):
                 parallel_overscan_size=parallel_overscan_size,
             )
 
-    @classmethod
-    def top_left(
-        cls,
-        array_electrons,
-        parallel_size=2086,
-        serial_size=2128,
-        serial_prescan_size=51,
-        serial_overscan_size=29,
-        parallel_overscan_size=20,
-    ):
-        """
-        Use an input array of a Euclid quadrant corresponding to the top-left of a Euclid CCD and rotate the quadrant
-        to the correct orientation for arCTIc clocking.
-
-        See the docstring of the `FrameEuclid` class for a complete description of the Euclid FPA, quadrants and
-        rotations.
-        """
-
-        scans = ScansEuclid.top_left(
-            parallel_size=parallel_size,
-            serial_size=serial_size,
-            serial_prescan_size=serial_prescan_size,
-            serial_overscan_size=serial_overscan_size,
-            parallel_overscan_size=parallel_overscan_size,
-        )
-
-        return cls.manual(
-            array=array_electrons, pixel_scales=0.1, roe_corner=(0, 0), scans=scans
-        )
-
-    @classmethod
-    def top_right(
-        cls,
-        array,
-        parallel_size=2086,
-        serial_size=2128,
-        parallel_overscan_size=20,
-        serial_prescan_size=51,
-        serial_overscan_size=29,
-    ):
-        """
-        Use an input array of a Euclid quadrant corresponding the top-left of a Euclid CCD and rotate the  quadrant to
-        the correct orientation for arCTIc clocking.
-
-        See the docstring of the `FrameEuclid` class for a complete description of the Euclid FPA, quadrants and
-        rotations.
-        """
-
-        scans = ScansEuclid.top_right(
-            parallel_size=parallel_size,
-            serial_size=serial_size,
-            serial_prescan_size=serial_prescan_size,
-            serial_overscan_size=serial_overscan_size,
-            parallel_overscan_size=parallel_overscan_size,
-        )
-
-        return cls.manual(array=array, pixel_scales=0.1, roe_corner=(0, 1), scans=scans)
-
-    @classmethod
-    def bottom_left(
-        cls,
-        array,
-        parallel_size=2086,
-        serial_size=2128,
-        serial_prescan_size=51,
-        serial_overscan_size=29,
-        parallel_overscan_size=20,
-    ):
-        """
-        Use an input array of a Euclid quadrant corresponding to the bottom-left of a Euclid CCD and rotate the
-        quadrant to the correct orientation for arCTIc clocking.
-
-        See the docstring of the `FrameEuclid` class for a complete description of the Euclid FPA, quadrants and
-        rotations.
-        """
-
-        scans = ScansEuclid.bottom_left(
-            parallel_size=parallel_size,
-            serial_size=serial_size,
-            serial_prescan_size=serial_prescan_size,
-            serial_overscan_size=serial_overscan_size,
-            parallel_overscan_size=parallel_overscan_size,
-        )
-
-        return cls.manual(array=array, pixel_scales=0.1, roe_corner=(1, 0), scans=scans)
-
-    @classmethod
-    def bottom_right(
-        cls,
-        array,
-        parallel_size=2086,
-        serial_size=2128,
-        serial_prescan_size=51,
-        serial_overscan_size=29,
-        parallel_overscan_size=20,
-    ):
-        """
-        Use an input array of a Euclid quadrant corresponding to the bottom-right of a Euclid CCD and rotate the
-        quadrant to the correct orientation for arCTIc clocking.
-
-        See the docstring of the `FrameEuclid` class for a complete description of the Euclid FPA, quadrants and
-        rotations.
-        """
-
-        scans = ScansEuclid.bottom_right(
-            parallel_size=parallel_size,
-            serial_size=serial_size,
-            serial_prescan_size=serial_prescan_size,
-            serial_overscan_size=serial_overscan_size,
-            parallel_overscan_size=parallel_overscan_size,
-        )
-
-        return cls.manual(array=array, pixel_scales=0.1, roe_corner=(1, 1), scans=scans)
-
-
-class ScansEuclid(abstract_frame.Scans):
     @classmethod
     def top_left(
         cls,
@@ -354,12 +324,16 @@ class ScansEuclid(abstract_frame.Scans):
             )
         )
 
-        return abstract_frame.Scans(
+        layout_2d = Layout2DEuclid(
+            shape_2d=(parallel_size, serial_size),
+            original_roe_corner=(0, 0),
             parallel_overscan=parallel_overscan,
             serial_prescan=serial_prescan,
             serial_overscan=serial_overscan,
         )
 
+        return layout_2d.new_rotated_from_roe_corner(roe_corner=(0, 0))
+
     @classmethod
     def top_right(
         cls,
@@ -392,12 +366,16 @@ class ScansEuclid(abstract_frame.Scans):
             (0, parallel_size - parallel_overscan_size, 0, serial_overscan_size)
         )
 
-        return abstract_frame.Scans(
+        layout_2d = Layout2DEuclid(
+            shape_2d=(parallel_size, serial_size),
+            original_roe_corner=(0, 1),
             parallel_overscan=parallel_overscan,
             serial_prescan=serial_prescan,
             serial_overscan=serial_overscan,
         )
 
+        return layout_2d.new_rotated_from_roe_corner(roe_corner=(0, 1))
+
     @classmethod
     def bottom_left(
         cls,
@@ -433,12 +411,16 @@ class ScansEuclid(abstract_frame.Scans):
             )
         )
 
-        return abstract_frame.Scans(
+        layout_2d = Layout2DEuclid(
+            shape_2d=(parallel_size, serial_size),
+            original_roe_corner=(1, 0),
             parallel_overscan=parallel_overscan,
             serial_prescan=serial_prescan,
             serial_overscan=serial_overscan,
         )
 
+        return layout_2d.new_rotated_from_roe_corner(roe_corner=(1, 0))
+
     @classmethod
     def bottom_right(
         cls,
@@ -471,8 +453,12 @@ class ScansEuclid(abstract_frame.Scans):
             (0, parallel_size - parallel_overscan_size, 0, serial_overscan_size)
         )
 
-        return abstract_frame.Scans(
+        layout_2d = Layout2DEuclid(
+            shape_2d=(parallel_size, serial_size),
+            original_roe_corner=(1, 1),
             parallel_overscan=parallel_overscan,
             serial_prescan=serial_prescan,
             serial_overscan=serial_overscan,
         )
+
+        return layout_2d.new_rotated_from_roe_corner(roe_corner=(1, 1))
