@@ -2,13 +2,14 @@ import numpy as np
 
 from autoconf import conf
 from autoarray import exc
-from autoarray.structures import arrays
-from autoarray.structures import grids
+from autoarray.structures.arrays.two_d import array_2d
+from autoarray.structures.grids.two_d import grid_2d
+from autoarray.structures.grids.two_d import grid_2d_irregular
 from autoarray.structures import visibilities as vis
 from autoarray.operators import convolver as conv, transformer as trans
-from autoarray.inversion import regularization as reg, mappers
+from autoarray.inversion import regularization as reg, mappers, inversion_util
 from autoarray.dataset import imaging, interferometer
-from autoarray.util import inversion_util
+from autoarray import preloads as pload
 from scipy.interpolate import griddata
 from scipy import sparse
 import pylops
@@ -29,44 +30,31 @@ class SettingsInversion:
         self.maxiter = maxiter
         self.check_solution = check_solution
 
-    @property
-    def tag(self):
-        return (
-            f"{conf.instance['notation']['settings_tags']['inversion']['inversion']}["
-            f"{self.use_linear_operators_tag}]"
-        )
-
-    @property
-    def use_linear_operators_tag(self):
-        if not self.use_linear_operators:
-            return f"{conf.instance['notation']['settings_tags']['inversion']['use_matrices']}"
-        return f"{conf.instance['notation']['settings_tags']['inversion']['use_linear_operators']}"
-
 
 def inversion(
-    masked_dataset,
+    dataset,
     mapper: typing.Union[mappers.MapperRectangular, mappers.MapperVoronoi],
     regularization,
     settings=SettingsInversion(),
 ):
 
-    if isinstance(masked_dataset, imaging.MaskedImaging):
+    if isinstance(dataset, imaging.Imaging):
 
         return InversionImagingMatrix.from_data_mapper_and_regularization(
-            image=masked_dataset.image,
-            noise_map=masked_dataset.noise_map,
-            convolver=masked_dataset.convolver,
+            image=dataset.image,
+            noise_map=dataset.noise_map,
+            convolver=dataset.convolver,
             mapper=mapper,
             regularization=regularization,
             settings=settings,
         )
 
-    elif isinstance(masked_dataset, interferometer.MaskedInterferometer):
+    elif isinstance(dataset, interferometer.Interferometer):
 
         return AbstractInversionInterferometer.from_data_mapper_and_regularization(
-            visibilities=masked_dataset.visibilities,
-            noise_map=masked_dataset.noise_map,
-            transformer=masked_dataset.transformer,
+            visibilities=dataset.visibilities,
+            noise_map=dataset.noise_map,
+            transformer=dataset.transformer,
             mapper=mapper,
             regularization=regularization,
             settings=settings,
@@ -127,7 +115,7 @@ class AbstractInversion:
 
         if shape_native is not None:
 
-            grid = grids.Grid2D.bounding_box(
+            grid = grid_2d.Grid2D.bounding_box(
                 bounding_box=self.mapper.source_pixelization_grid.extent,
                 shape_native=shape_native,
                 buffer_around_corners=False,
@@ -148,7 +136,7 @@ class AbstractInversion:
             dimension = int(np.sqrt(self.mapper.pixels))
             shape_native = (dimension, dimension)
 
-            grid = grids.Grid2D.bounding_box(
+            grid = grid_2d.Grid2D.bounding_box(
                 bounding_box=self.mapper.source_pixelization_grid.extent,
                 shape_native=shape_native,
                 buffer_around_corners=False,
@@ -164,13 +152,13 @@ class AbstractInversion:
         interpolated_reconstruction = griddata(
             points=self.mapper.source_pixelization_grid,
             values=values,
-            xi=grid.native_binned,
+            xi=grid.binned.native,
             method="linear",
         )
 
         interpolated_reconstruction[np.isnan(interpolated_reconstruction)] = 0.0
 
-        return arrays.Array2D.manual(
+        return array_2d.Array2D.manual(
             array=interpolated_reconstruction, pixel_scales=grid.pixel_scales
         )
 
@@ -202,7 +190,7 @@ class AbstractInversion:
 
     @property
     def brightest_reconstruction_pixel_centre(self):
-        return grids.Grid2DIrregular(
+        return grid_2d_irregular.Grid2DIrregular(
             grid=[
                 self.mapper.source_pixelization_grid[
                     self.brightest_reconstruction_pixel
@@ -235,9 +223,13 @@ class AbstractInversion:
 
 class AbstractInversionMatrix:
     def __init__(
-        self, curvature_reg_matrix: np.ndarray, regularization_matrix: np.ndarray
+        self,
+        curvature_reg_matrix: np.ndarray,
+        curvature_matrix: np.ndarray,
+        regularization_matrix: np.ndarray,
     ):
 
+        self.curvature_matrix = curvature_matrix
         self.curvature_reg_matrix = curvature_reg_matrix
         self.regularization_matrix = regularization_matrix
 
@@ -257,12 +249,13 @@ class AbstractInversionMatrix:
 class InversionImagingMatrix(AbstractInversion, AbstractInversionMatrix):
     def __init__(
         self,
-        image: arrays.Array2D,
-        noise_map: arrays.Array2D,
+        image: array_2d.Array2D,
+        noise_map: array_2d.Array2D,
         convolver: conv.Convolver,
         mapper: typing.Union[mappers.MapperRectangular, mappers.MapperVoronoi],
         regularization: reg.Regularization,
         blurred_mapping_matrix: np.ndarray,
+        curvature_matrix: np.ndarray,
         regularization_matrix: np.ndarray,
         curvature_reg_matrix: np.ndarray,
         reconstruction: np.ndarray,
@@ -315,6 +308,7 @@ class InversionImagingMatrix(AbstractInversion, AbstractInversionMatrix):
 
         AbstractInversionMatrix.__init__(
             self=self,
+            curvature_matrix=curvature_matrix,
             curvature_reg_matrix=curvature_reg_matrix,
             regularization_matrix=regularization_matrix,
         )
@@ -326,17 +320,24 @@ class InversionImagingMatrix(AbstractInversion, AbstractInversionMatrix):
     @classmethod
     def from_data_mapper_and_regularization(
         cls,
-        image: arrays.Array2D,
-        noise_map: arrays.Array2D,
+        image: array_2d.Array2D,
+        noise_map: array_2d.Array2D,
         convolver: conv.Convolver,
         mapper: typing.Union[mappers.MapperRectangular, mappers.MapperVoronoi],
         regularization: reg.Regularization,
         settings=SettingsInversion(),
+        preloads=pload.Preloads(),
     ):
 
-        blurred_mapping_matrix = convolver.convolve_mapping_matrix(
-            mapping_matrix=mapper.mapping_matrix
-        )
+        if preloads.blurred_mapping_matrix is None:
+
+            blurred_mapping_matrix = convolver.convolve_mapping_matrix(
+                mapping_matrix=mapper.mapping_matrix
+            )
+
+        else:
+
+            blurred_mapping_matrix = preloads.blurred_mapping_matrix
 
         data_vector = inversion_util.data_vector_via_blurred_mapping_matrix_from(
             blurred_mapping_matrix=blurred_mapping_matrix,
@@ -344,9 +345,24 @@ class InversionImagingMatrix(AbstractInversion, AbstractInversionMatrix):
             noise_map=noise_map,
         )
 
-        curvature_matrix = inversion_util.curvature_matrix_via_mapping_matrix_from(
-            mapping_matrix=blurred_mapping_matrix, noise_map=noise_map
-        )
+        if preloads.curvature_matrix_sparse_preload is None:
+
+            curvature_matrix = inversion_util.curvature_matrix_via_mapping_matrix_from(
+                mapping_matrix=blurred_mapping_matrix, noise_map=noise_map
+            )
+
+        else:
+
+            curvature_matrix = inversion_util.curvature_matrix_via_sparse_preload_from(
+                mapping_matrix=blurred_mapping_matrix,
+                noise_map=noise_map,
+                curvature_matrix_sparse_preload=preloads.curvature_matrix_sparse_preload.astype(
+                    "int"
+                ),
+                curvature_matrix_preload_counts=preloads.curvature_matrix_preload_counts.astype(
+                    "int"
+                ),
+            )
 
         regularization_matrix = regularization.regularization_matrix_from_mapper(
             mapper=mapper
@@ -371,6 +387,7 @@ class InversionImagingMatrix(AbstractInversion, AbstractInversionMatrix):
             mapper=mapper,
             regularization=regularization,
             blurred_mapping_matrix=blurred_mapping_matrix,
+            curvature_matrix=curvature_matrix,
             regularization_matrix=regularization_matrix,
             curvature_reg_matrix=curvature_reg_matrix,
             reconstruction=values,
@@ -384,10 +401,8 @@ class InversionImagingMatrix(AbstractInversion, AbstractInversionMatrix):
             reconstruction=self.reconstruction,
         )
 
-        return arrays.Array2D(
-            array=reconstructed_image,
-            mask=self.mapper.source_grid_slim.mask.mask_sub_1,
-            store_slim=True,
+        return array_2d.Array2D(
+            array=reconstructed_image, mask=self.mapper.source_grid_slim.mask.mask_sub_1
         )
 
     @property
@@ -419,6 +434,24 @@ class InversionImagingMatrix(AbstractInversion, AbstractInversionMatrix):
             all_sub_slim_indexes_for_pixelization_index=self.mapper.all_sub_slim_indexes_for_pixelization_index,
         )
 
+    @property
+    def curvature_matrix_sparse_preload(self):
+
+        curvature_matrix_sparse_preload, curvature_matrix_preload_counts = inversion_util.curvature_matrix_sparse_preload_via_mapping_matrix_from(
+            mapping_matrix=self.blurred_mapping_matrix
+        )
+
+        return curvature_matrix_sparse_preload
+
+    @property
+    def curvature_matrix_preload_counts(self):
+
+        curvature_matrix_sparse_preload, curvature_matrix_preload_counts = inversion_util.curvature_matrix_sparse_preload_via_mapping_matrix_from(
+            mapping_matrix=self.blurred_mapping_matrix
+        )
+
+        return curvature_matrix_preload_counts
+
 
 class AbstractInversionInterferometer(AbstractInversion):
     def __init__(
@@ -483,12 +516,23 @@ class AbstractInversionInterferometer(AbstractInversion):
             reconstruction=self.reconstruction,
         )
 
-        return arrays.Array2D(
+        return array_2d.Array2D(
             array=mapped_reconstructed_image,
             mask=self.mapper.source_grid_slim.mask.mask_sub_1,
-            store_slim=True,
         )
 
+    @property
+    def residual_map(self):
+        return None
+
+    @property
+    def normalized_residual_map(self):
+        return None
+
+    @property
+    def chi_squared_map(self):
+        return None
+
 
 class InversionInterferometerMatrix(
     AbstractInversionInterferometer, AbstractInversionMatrix
@@ -503,6 +547,7 @@ class InversionInterferometerMatrix(
         regularization_matrix: np.ndarray,
         reconstruction: np.ndarray,
         transformed_mapping_matrix: np.ndarray,
+        curvature_matrix: np.ndarray,
         curvature_reg_matrix: np.ndarray,
         settings: SettingsInversion,
     ):
@@ -555,6 +600,7 @@ class InversionInterferometerMatrix(
 
         AbstractInversionMatrix.__init__(
             self=self,
+            curvature_matrix=curvature_matrix,
             curvature_reg_matrix=curvature_reg_matrix,
             regularization_matrix=regularization_matrix,
         )
@@ -613,6 +659,7 @@ class InversionInterferometerMatrix(
             noise_map=noise_map,
             transformer=transformer,
             mapper=mapper,
+            curvature_matrix=curvature_matrix,
             regularization=regularization,
             transformed_mapping_matrix=transformed_mapping_matrix,
             regularization_matrix=regularization_matrix,
@@ -709,9 +756,7 @@ class InversionInterferometerLinearOperator(AbstractInversionInterferometer):
             mapper=mapper
         )
 
-        Aop = pylops.MatrixMult(
-            sparse.bsr_matrix(mapper.mapping_matrix), dtype="float64"
-        )
+        Aop = pylops.MatrixMult(sparse.bsr_matrix(mapper.mapping_matrix))
 
         Fop = transformer
 
@@ -726,9 +771,7 @@ class InversionInterferometerLinearOperator(AbstractInversionInterferometer):
 
         preconditioner_inverse_matrix = np.linalg.inv(preconditioner_matrix)
 
-        MOp = pylops.MatrixMult(
-            sparse.bsr_matrix(preconditioner_inverse_matrix), dtype="float64"
-        )
+        MOp = pylops.MatrixMult(sparse.bsr_matrix(preconditioner_inverse_matrix))
 
         log_det_curvature_reg_matrix_term = 2.0 * np.sum(
             np.log(np.diag(np.linalg.cholesky(preconditioner_matrix)))
@@ -739,14 +782,11 @@ class InversionInterferometerLinearOperator(AbstractInversionInterferometer):
             Regs=None,
             epsNRs=[1.0],
             data=visibilities.ordered_1d,
-            Weight=pylops.Diagonal(diag=noise_map.weights_ordered_1d, dtype="float64"),
-            NRegs=[
-                pylops.MatrixMult(
-                    sparse.bsr_matrix(regularization_matrix), dtype="float64"
-                )
-            ],
+            Weight=pylops.Diagonal(diag=noise_map.weights_ordered_1d),
+            NRegs=[pylops.MatrixMult(sparse.bsr_matrix(regularization_matrix))],
             M=MOp,
             tol=settings.tolerance,
+            atol=settings.tolerance,
             **dict(maxiter=settings.maxiter),
         )
 
@@ -772,3 +812,7 @@ class InversionInterferometerLinearOperator(AbstractInversionInterferometer):
         return self.transformer.visibilities_from_image(
             image=self.mapped_reconstructed_image
         )
+
+    @property
+    def errors(self):
+        return None
