@@ -1,7 +1,8 @@
 import numpy as np
 from autoconf import conf
 from autoarray import exc
-from autoarray.structures import grids
+from autoarray.structures.grids.two_d import grid_2d
+from autoarray.structures.grids.two_d import grid_2d_pixelization
 from autoarray.inversion import mappers
 
 import copy
@@ -14,91 +15,35 @@ class SettingsPixelization:
         pixel_limit: int = None,
         is_stochastic: bool = False,
         kmeans_seed: int = 0,
-        preload_sparse_grids_of_planes=None,
     ):
 
         self.use_border = use_border
         self.pixel_limit = pixel_limit
         self.is_stochastic = is_stochastic
         self.kmeans_seed = kmeans_seed
-        self.preload_sparse_grids_of_planes = preload_sparse_grids_of_planes
-
-    @property
-    def tag(self):
-        return (
-            f"{conf.instance['notation']['settings_tags']['pixelization']['pixelization']}["
-            f"{self.use_border_tag}"
-            f"{self.is_stochastic_tag}]"
-        )
-
-    @property
-    def use_border_tag(self):
-        """Generate a tag for whether a border is used by the inversion.
-
-        This changes the setup folder as follows (the example tags below are the default config tags):
-
-        inversion_use_border = `False` -> settings
-        inversion_use_border = `True` -> settings___no_border
-        """
-        if self.use_border:
-
-            tag = conf.instance["notation"]["settings_tags"]["pixelization"][
-                "use_border"
-            ]
-
-            if not tag:
-                return str(tag)
-            return f"{tag}"
-        elif not self.use_border:
-            return f"{conf.instance['notation']['settings_tags']['pixelization']['no_border']}"
-
-    @property
-    def is_stochastic_tag(self):
-        """Generate a tag for whether the inversion is stochastic.
-
-        This changes the setup folder as follows (the example tags below are the default config tags):
-
-        inversion_stochastic = `False` -> settings
-        inversion_stochastic = `True` -> settings___stochastic
-        """
-        if not self.is_stochastic:
-
-            tag = conf.instance["notation"]["settings_tags"]["pixelization"][
-                "not_stochastic"
-            ]
-            if not tag:
-                return tag
-            return "__" + tag
-        elif self.is_stochastic:
-            return (
-                "__"
-                + conf.instance["notation"]["settings_tags"]["pixelization"][
-                    "stochastic"
-                ]
-            )
 
     def settings_with_is_stochastic_true(self):
         settings = copy.copy(self)
         settings.is_stochastic = True
         return settings
 
-    def modify_preload(self, preload_sparse_grids_of_planes):
-
-        settings = copy.copy(self)
-        settings.preload_sparse_grids_of_planes = preload_sparse_grids_of_planes
-        return settings
-
 
 class Pixelization:
     def __init__(self):
         """ Abstract base class for a pixelization, which discretizes grid of (y,x) coordinates into pixels.
         """
 
-    def mapper_from_grid_and_sparse_grid(self, grid: grids.Grid2D, border: np.ndarray):
+    def mapper_from_grid_and_sparse_grid(
+        self, grid: grid_2d.Grid2D, border: np.ndarray
+    ):
         raise NotImplementedError(
             "pixelization_mapper_from_grids_and_borders should be overridden"
         )
 
+    def weight_map_from_hyper_image(self, hyper_image: np.ndarray):
+
+        raise NotImplementedError()
+
     def __str__(self):
         return "\n".join(["{}: {}".format(k, v) for k, v in self.__dict__.items()])
 
@@ -130,13 +75,14 @@ class Rectangular(Pixelization):
 
     def mapper_from_grid_and_sparse_grid(
         self,
-        grid: grids.Grid2D,
-        sparse_grid: grids.Grid2D = None,
-        sparse_image_plane_grid: grids.Grid2D = None,
+        grid: grid_2d.Grid2D,
+        sparse_grid: grid_2d.Grid2D = None,
+        sparse_image_plane_grid: grid_2d.Grid2D = None,
         hyper_image: np.ndarray = None,
         settings=SettingsPixelization(),
     ):
-        """Setup a rectangular mapper from a rectangular pixelization, as follows:
+        """
+        Setup a rectangular mapper from a rectangular pixelization, as follows:
 
         1) If a border is supplied, relocate all of the grid's and sub grid pixels beyond the border.
         2) Determine the rectangular pixelization's geometry, by laying the pixelization over the sub-grid.
@@ -157,7 +103,7 @@ class Rectangular(Pixelization):
         else:
             relocated_grid = grid
 
-        pixelization_grid = grids.Grid2DRectangular.overlay_grid(
+        pixelization_grid = grid_2d_pixelization.Grid2DRectangular.overlay_grid(
             shape_native=self.shape, grid=relocated_grid
         )
 
@@ -169,7 +115,7 @@ class Rectangular(Pixelization):
 
     def sparse_grid_from_grid(
         self,
-        grid: grids.Grid2D,
+        grid: grid_2d.Grid2D,
         hyper_image: np.ndarray = None,
         settings=SettingsPixelization(),
     ):
@@ -188,9 +134,9 @@ class Voronoi(Pixelization):
 
     def mapper_from_grid_and_sparse_grid(
         self,
-        grid: grids.Grid2D,
-        sparse_grid: grids.Grid2DSparse = None,
-        sparse_image_plane_grid: grids.Grid2DSparse = None,
+        grid: grid_2d.Grid2D,
+        sparse_grid: grid_2d.Grid2DSparse = None,
+        sparse_image_plane_grid: grid_2d.Grid2DSparse = None,
         hyper_image: np.ndarray = None,
         settings=SettingsPixelization(),
     ):
@@ -217,17 +163,20 @@ class Voronoi(Pixelization):
         """
 
         if settings.use_border:
+
             relocated_grid = grid.relocated_grid_from_grid(grid=grid)
+
             relocated_pixelization_grid = grid.relocated_pixelization_grid_from_pixelization_grid(
                 pixelization_grid=sparse_grid
             )
+
         else:
             relocated_grid = grid
             relocated_pixelization_grid = sparse_grid
 
         try:
 
-            pixelization_grid = grids.Grid2DVoronoi(
+            pixelization_grid = grid_2d_pixelization.Grid2DVoronoi(
                 grid=relocated_pixelization_grid,
                 nearest_pixelization_index_for_slim_index=sparse_grid.sparse_index_for_slim_index,
             )
@@ -260,12 +209,12 @@ class VoronoiMagnification(Voronoi):
 
     def sparse_grid_from_grid(
         self,
-        grid: grids.Grid2D,
+        grid: grid_2d.Grid2D,
         hyper_image: np.ndarray = None,
         settings=SettingsPixelization(),
     ):
 
-        return grids.Grid2DSparse.from_grid_and_unmasked_2d_grid_shape(
+        return grid_2d.Grid2DSparse.from_grid_and_unmasked_2d_grid_shape(
             grid=grid, unmasked_sparse_shape=self.shape
         )
 
@@ -294,14 +243,14 @@ class VoronoiBrightnessImage(Voronoi):
 
     def sparse_grid_from_grid(
         self,
-        grid: grids.Grid2D,
+        grid: grid_2d.Grid2D,
         hyper_image: np.ndarray,
         settings=SettingsPixelization(),
     ):
 
         weight_map = self.weight_map_from_hyper_image(hyper_image=hyper_image)
 
-        return grids.Grid2DSparse.from_total_pixels_grid_and_weight_map(
+        return grid_2d.Grid2DSparse.from_total_pixels_grid_and_weight_map(
             total_pixels=self.pixels,
             grid=grid,
             weight_map=weight_map,
