@@ -2,8 +2,6 @@ from autoarray.plot.mat_wrap.wrap import wrap_base
 
 wrap_base.set_backend()
 
-from autoarray.plot.mat_wrap import wrap
-
 import matplotlib.pyplot as plt
 from matplotlib import patches as ptch
 from matplotlib.collections import PatchCollection
@@ -13,7 +11,9 @@ import typing
 from typing import List, Union
 
 from autoarray.inversion import mappers
-from autoarray.structures import grids, vector_fields
+from autoarray.structures.grids.two_d import grid_2d
+from autoarray.structures.grids.two_d import grid_2d_irregular
+from autoarray.structures.vector_fields import vector_field_irregular
 from autoarray import exc
 
 
@@ -77,7 +77,7 @@ class GridScatter(AbstractMatWrap2D):
         list of colors can be specified which the plot cycles through.
     """
 
-    def scatter_grid(self, grid: typing.Union[np.ndarray, grids.Grid2D]):
+    def scatter_grid(self, grid: typing.Union[np.ndarray, grid_2d.Grid2D]):
         """
         Plot an input grid of (y,x) coordinates using the matplotlib method `plt.scatter`.
 
@@ -88,15 +88,18 @@ class GridScatter(AbstractMatWrap2D):
         """
 
         config_dict = self.config_dict
-        config_dict.pop("c")
+
+        if len(config_dict["c"]) > 1:
+            config_dict["c"] = config_dict["c"][0]
 
         try:
-            plt.scatter(y=np.asarray(grid)[:, 0], x=np.asarray(grid)[:, 1], **config_dict)
-        except IndexError:
+            plt.scatter(y=grid[:, 0], x=grid[:, 1], **config_dict)
+        except (IndexError, TypeError):
             return self.scatter_grid_list(grid_list=grid)
 
     def scatter_grid_list(
-        self, grid_list: Union[List[grids.Grid2D], List[grids.Grid2DIrregular]]
+        self,
+        grid_list: Union[List[grid_2d.Grid2D], List[grid_2d_irregular.Grid2DIrregular]],
     ):
         """
          Plot an input list of grids of (y,x) coordinates using the matplotlib method `plt.scatter`.
@@ -119,18 +122,13 @@ class GridScatter(AbstractMatWrap2D):
         try:
             for grid in grid_list:
 
-                plt.scatter(
-                    y=np.asarray(grid)[:, 0],
-                    x=np.asarray(grid)[:, 1],
-                    c=next(color),
-                    **config_dict,
-                )
+                plt.scatter(y=grid[:, 0], x=grid[:, 1], c=next(color), **config_dict)
         except IndexError:
             return None
 
     def scatter_grid_colored(
         self,
-        grid: typing.Union[np.ndarray, grids.Grid2D],
+        grid: typing.Union[np.ndarray, grid_2d.Grid2D],
         color_array: np.ndarray,
         cmap: str,
     ):
@@ -152,16 +150,10 @@ class GridScatter(AbstractMatWrap2D):
         config_dict = self.config_dict
         config_dict.pop("c")
 
-        plt.scatter(
-            y=np.asarray(grid)[:, 0],
-            x=np.asarray(grid)[:, 1],
-            c=color_array,
-            cmap=cmap,
-            **config_dict,
-        )
+        plt.scatter(y=grid[:, 0], x=grid[:, 1], c=color_array, cmap=cmap, **config_dict)
 
     def scatter_grid_indexes(
-        self, grid: typing.Union[np.ndarray, grids.Grid2D], indexes: np.ndarray
+        self, grid: typing.Union[np.ndarray, grid_2d.Grid2D], indexes: np.ndarray
     ):
         """
         Plot specific points of an input grid of (y,x) coordinates, which are specified according to the 1D or 2D
@@ -203,8 +195,8 @@ class GridScatter(AbstractMatWrap2D):
             ):
 
                 plt.scatter(
-                    y=np.asarray(grid[index_list, 0]),
-                    x=np.asarray(grid[index_list, 1]),
+                    y=grid[index_list, 0],
+                    x=grid[index_list, 1],
                     color=next(color),
                     **config_dict,
                 )
@@ -216,8 +208,8 @@ class GridScatter(AbstractMatWrap2D):
                 ys, xs = map(list, zip(*index_list))
 
                 plt.scatter(
-                    y=np.asarray(grid.native[ys, xs, 0]),
-                    x=np.asarray(grid.native[ys, xs, 1]),
+                    y=grid.native[ys, xs, 0],
+                    x=grid.native[ys, xs, 1],
                     color=next(color),
                     **config_dict,
                 )
@@ -275,7 +267,7 @@ class GridPlot(AbstractMatWrap2D):
         for y in ys:
             plt.plot([xs[0], xs[-1]], [y, y], **self.config_dict)
 
-    def plot_grid(self, grid: typing.Union[np.ndarray, grids.Grid2D]):
+    def plot_grid(self, grid: typing.Union[np.ndarray, grid_2d.Grid2D]):
         """
         Plot an input grid of (y,x) coordinates using the matplotlib method `plt.scatter`.
 
@@ -285,12 +277,13 @@ class GridPlot(AbstractMatWrap2D):
             The grid of (y,x) coordinates that is plotted.
         """
         try:
-            plt.plot(np.asarray(grid)[:, 1], np.asarray(grid)[:, 0], **self.config_dict)
-        except IndexError:
+            plt.plot(grid[:, 1], grid[:, 0], **self.config_dict)
+        except (IndexError, TypeError):
             return self.plot_grid_list(grid_list=grid)
 
     def plot_grid_list(
-        self, grid_list: Union[List[grids.Grid2D], List[grids.Grid2DIrregular]]
+        self,
+        grid_list: Union[List[grid_2d.Grid2D], List[grid_2d_irregular.Grid2DIrregular]],
     ):
         """
          Plot an input list of grids of (y,x) coordinates using the matplotlib method `plt.line`.
@@ -316,12 +309,7 @@ class GridPlot(AbstractMatWrap2D):
         try:
             for grid in grid_list:
 
-                plt.plot(
-                    np.asarray(grid)[:, 1],
-                    np.asarray(grid)[:, 0],
-                    c=next(color),
-                    **config_dict,
-                )
+                plt.plot(grid[:, 1], grid[:, 0], c=next(color), **config_dict)
         except IndexError:
             return None
 
@@ -337,7 +325,9 @@ class VectorFieldQuiver(AbstractMatWrap2D):
     https://matplotlib.org/3.3.2/api/_as_gen/matplotlib.pyplot.quiver.html
     """
 
-    def quiver_vector_field(self, vector_field: vector_fields.VectorField2DIrregular):
+    def quiver_vector_field(
+        self, vector_field: vector_field_irregular.VectorField2DIrregular
+    ):
         """
          Plot a vector field using the matplotlib method `plt.quiver` such that each vector appears as an arrow whose
          direction depends on the y and x magnitudes of the vector.
@@ -377,6 +367,7 @@ class PatchOverlay(AbstractMatWrap2D):
         patches : [Patch]
             The patches that are laid over the figure.
         """
+
         patch_collection = PatchCollection(patches=patches, **self.config_dict)
 
         plt.gcf().gca().add_collection(patch_collection)
@@ -398,9 +389,9 @@ class VoronoiDrawer(AbstractMatWrap2D):
         self,
         mapper: mappers.MapperVoronoi,
         values: np.ndarray,
-        cmap: wrap.Cmap,
-        colorbar: wrap.Colorbar,
-        colorbar_tickparams: wrap.ColorbarTickParams = None,
+        cmap: wrap_base.Cmap,
+        colorbar: wrap_base.Colorbar,
+        colorbar_tickparams: wrap_base.ColorbarTickParams = None,
     ):
         """
         Draws the Voronoi pixels of the input `mapper` using its `pixelization_grid` which contains the (y,x) 
@@ -427,14 +418,21 @@ class VoronoiDrawer(AbstractMatWrap2D):
             color_values = np.where(values > vmax, vmax, values)
             color_values = np.where(values < vmin, vmin, color_values)
 
-            color_array = (color_values - vmin) / (vmax - vmin)
+            if vmax != vmin:
+                color_array = (color_values - vmin) / (vmax - vmin)
+            else:
+                color_array = np.ones(color_values.shape[0])
 
             cmap = plt.get_cmap(cmap.config_dict["cmap"])
-            colorbar = colorbar.set_with_color_values(
-                cmap=cmap, color_values=color_values
-            )
-            if colorbar is not None and colorbar_tickparams is not None:
-                colorbar_tickparams.set(cb=colorbar)
+
+            if colorbar is not None:
+
+                colorbar = colorbar.set_with_color_values(
+                    cmap=cmap, color_values=color_values
+                )
+                if colorbar is not None and colorbar_tickparams is not None:
+                    colorbar_tickparams.set(cb=colorbar)
+
         else:
             cmap = plt.get_cmap("Greys")
             color_array = np.zeros(shape=mapper.pixels)
