@@ -1,6 +1,8 @@
 from autoconf import conf
 import matplotlib
 
+from autoarray.structures.arrays.two_d import array_2d
+
 
 def set_backend():
 
@@ -27,7 +29,7 @@ import os
 import typing
 
 
-from autoarray.structures import abstract_structure, arrays
+from autoarray.structures import abstract_structure
 from autoarray import exc
 
 
@@ -43,7 +45,7 @@ class Units:
 
         1) Species the units of the plot (e.g. meters, kilometers) and contains a conversion factor which converts
            the plotted data from its current units (e.g. meters) to the units plotted (e.g. kilometeters). Pixel units
-            can be used if `use_scaled=False`.
+           can be used if `use_scaled=False`.
 
         2) Uses the conversion above to manually override the yticks and xticks of the figure, so it appears in the
            converted units.
@@ -499,7 +501,7 @@ class AbstractTicks(AbstractMatWrap):
             return np.linspace(min_value, max_value, 5)
 
     def tick_values_in_units_from(
-        self, array: arrays.Array2D, min_value: float, max_value: float, units: Units
+        self, array: array_2d.Array2D, min_value: float, max_value: float, units: Units
     ) -> typing.Optional[np.ndarray]:
         """
         Calculate the labels used for the yticks or xticks from input values of the minimum and maximum coordinate
@@ -509,7 +511,7 @@ class AbstractTicks(AbstractMatWrap):
 
         Parameters
         ----------
-        array : arrays.Array2D
+        array : array_2d.Array2D
             The array of data that is to be plotted, whose 2D shape is used to determine the tick values in units of
             pixels if this is the units specified by `units`.
         min_value : float
@@ -545,7 +547,7 @@ class AbstractTicks(AbstractMatWrap):
 class YTicks(AbstractTicks):
     def set(
         self,
-        array: typing.Optional[arrays.Array2D],
+        array: typing.Optional[array_2d.Array2D],
         min_value: float,
         max_value: float,
         units: Units,
@@ -555,7 +557,7 @@ class YTicks(AbstractTicks):
 
         Parameters
         -----------
-        array : arrays.Array2D
+        array : array_2d.Array2D
             The 2D array of data which is plotted.
         min_value : float
             the minimum value of the yticks that figure is plotted using.
@@ -575,7 +577,7 @@ class YTicks(AbstractTicks):
 class XTicks(AbstractTicks):
     def set(
         self,
-        array: typing.Optional[arrays.Array2D],
+        array: typing.Optional[array_2d.Array2D],
         min_value: float,
         max_value: float,
         units: Units,
@@ -585,7 +587,7 @@ class XTicks(AbstractTicks):
 
         Parameters
         -----------
-        array : arrays.Array2D
+        array : array_2d.Array2D
             The 2D array of data which is plotted.
         min_value : float
             the minimum value of the xticks that figure is plotted using.
@@ -773,6 +775,7 @@ class Legend(AbstractMatWrap):
 
             config_dict = self.config_dict
             config_dict.pop("include") if "include" in config_dict else None
+            config_dict.pop("include_2d") if "include_2d" in config_dict else None
 
             plt.legend(**config_dict)
 
@@ -849,6 +852,10 @@ class Output:
                 plt.savefig(
                     path.join(self.path, f"{filename}.png"), bbox_inches="tight"
                 )
+            elif self.format == "pdf":
+                plt.savefig(
+                    path.join(self.path, f"{filename}.pdf"), bbox_inches="tight"
+                )
             elif self.format == "fits":
                 if structure is not None:
                     structure.output_to_fits(
@@ -865,6 +872,8 @@ class Output:
             plt.show()
         elif self.format == "png":
             plt.savefig(path.join(self.path, f"{filename}.png"), bbox_inches="tight")
+        elif self.format == "pdf":
+            plt.savefig(path.join(self.path, f"{filename}.pdf"), bbox_inches="tight")
 
 
 def remove_spaces_and_commas_from_colors(colors):
