@@ -6,11 +6,37 @@ import pickle
 class AbstractStructure(np.ndarray):
     def __array_finalize__(self, obj):
 
-        if isinstance(obj, AbstractStructure):
-            if hasattr(obj, "mask"):
-                self.mask = obj.mask
+        if hasattr(obj, "mask"):
+            self.mask = obj.mask
 
-    def _new_structure(self, grid, mask, store_slim):
+        if hasattr(obj, "exposure_info"):
+            self.exposure_info = obj.exposure_info
+
+    def __reduce__(self):
+
+        # Get the parent's __reduce__ tuple
+        pickled_state = super().__reduce__()
+
+        # Create our own tuple to pass to __setstate__
+        class_dict = {}
+
+        for key, value in self.__dict__.items():
+            class_dict[key] = value
+
+        new_state = pickled_state[2] + (class_dict,)
+
+        # Return a tuple that replaces the parent's __setstate__ tuple with our own
+        return pickled_state[0], pickled_state[1], new_state
+
+    # noinspection PyMethodOverriding
+    def __setstate__(self, state):
+
+        for key, value in state[-1].items():
+            setattr(self, key, value)
+
+        super().__setstate__(state[0:-1])
+
+    def _new_structure(self, structure: "AbstractStructure", mask):
         """Conveninence method for creating a new instance of the Grid2D class from this grid.
 
         This method is over-written by other grids (e.g. Grid2DIterate) such that the slim and native methods return
@@ -18,24 +44,20 @@ class AbstractStructure(np.ndarray):
 
         Parameters
         ----------
-        grid : np.ndarray or list
-            The (y,x) coordinates of the grid input as an ndarray of shape [total_sub_coordinates, 2] or list of lists.
+        data_structure : AbstractStructure
+            The structure which is to be turned into a new structure.
         mask : msk.Mask2D
-            The 2D mask associated with the grid, defining the pixels each grid coordinate is paired with and
-            originates from.
-        store_slim : bool
-            If True, the grid is stored in 1D as an ndarray of shape [total_unmasked_pixels, 2]. If False, it is
-            stored in 2D as an ndarray of shape [total_y_pixels, total_x_pixels, 2].
+            The mask associated with this structure.
         """
         raise NotImplementedError()
 
     @property
     def slim(self):
-        raise NotImplementedError
+        raise NotImplementedError()
 
     @property
     def native(self):
-        raise NotImplementedError
+        raise NotImplementedError()
 
     @property
     def shape_slim(self):
@@ -45,6 +67,14 @@ class AbstractStructure(np.ndarray):
     def sub_shape_slim(self):
         return self.mask.sub_shape_slim
 
+    @property
+    def shape_native(self):
+        return self.mask.shape
+
+    @property
+    def sub_shape_native(self):
+        return self.mask.sub_shape_native
+
     @property
     def pixel_scales(self):
         return self.mask.pixel_scales
@@ -69,9 +99,20 @@ class AbstractStructure(np.ndarray):
     def total_pixels(self):
         return self.shape[0]
 
-    @property
-    def binned_pixel_scales_from_bin_up_factor(self):
-        return self.mask.binned_pixel_scales_from_bin_up_factor
+    def resized_from(self, new_shape):
+        raise NotImplementedError
+
+    def padded_before_convolution_from(self, kernel_shape):
+        raise NotImplementedError
+
+    def trimmed_after_convolution_from(self, kernel_shape):
+        raise NotImplementedError
+
+    def structure_2d_from_result(self, result: np.ndarray):
+        raise NotImplementedError
+
+    def structure_2d_list_from_result_list(self, result_list: list):
+        raise NotImplementedError
 
     @classmethod
     def load(cls, file_path, filename):
@@ -85,18 +126,6 @@ class AbstractStructure(np.ndarray):
         with open(path.join(file_path, f"{filename}.pickle"), "wb") as f:
             pickle.dump(self, f)
 
-    def resized_from(self, new_shape):
-        raise NotImplementedError
-
-    def padded_before_convolution_from(self, kernel_shape):
-        raise NotImplementedError
-
-    def trimmed_after_convolution_from(self, kernel_shape):
-        raise NotImplementedError
-
-    def binned_up_from(self, bin_up_factor, method):
-        raise NotImplementedError
-
     def output_to_fits(self, file_path, overwrite):
         raise NotImplementedError
 
@@ -107,15 +136,6 @@ class AbstractStructure1D(AbstractStructure):
 
 
 class AbstractStructure2D(AbstractStructure):
-    def __array_finalize__(self, obj):
-
-        super().__array_finalize__(obj=obj)
-
-        if isinstance(obj, AbstractStructure2D):
-
-            if hasattr(obj, "store_slim"):
-                self.store_slim = obj.store_slim
-
     @property
     def native(self):
         raise NotImplementedError
