@@ -1,11 +1,9 @@
 import logging
 
 import numpy as np
-import pylops
 
-from autoarray.mask import mask_2d
-from autoarray.structures import grids
-from autoarray.util import array_util
+from autoarray.structures.grids.two_d import grid_2d_irregular
+from autoarray.structures.arrays.two_d import array_2d_util
 
 logging.basicConfig()
 logger = logging.getLogger(__name__)
@@ -93,19 +91,19 @@ class AbstractVisibilities(np.ndarray):
     @property
     def in_grid(self):
         """Returns the 1D complex NumPy array of values as an irregular grid."""
-        return grids.Grid2DIrregular(grid=self.in_array)
+        return grid_2d_irregular.Grid2DIrregular(grid=self.in_array)
 
     @property
     def shape_slim(self):
         return self.shape[0]
 
     @property
-    @array_util.Memoizer()
+    @array_2d_util.Memoizer()
     def amplitudes(self):
         return np.sqrt(np.square(self.real) + np.square(self.imag))
 
     @property
-    @array_util.Memoizer()
+    @array_2d_util.Memoizer()
     def phases(self):
         return np.arctan2(self.imag, self.real)
 
@@ -124,7 +122,7 @@ class AbstractVisibilities(np.ndarray):
         overwrite : bool
             If a file already exists at the path, if overwrite=True it is overwritten else an error is raised.
         """
-        array_util.numpy_array_2d_to_fits(
+        array_2d_util.numpy_array_2d_to_fits(
             array_2d=self.in_array, file_path=file_path, overwrite=overwrite
         )
 
@@ -141,7 +139,7 @@ class AbstractVisibilities(np.ndarray):
     @property
     def extent(self):
         """The extent of the visibilities if they are treated as a 2D grid in the complex plane."""
-        return np.asarray(
+        return np.array(
             [
                 self.scaled_minima[1],
                 self.scaled_maxima[1],
@@ -239,7 +237,7 @@ class Visibilities(AbstractVisibilities):
         hdu : int
             The Header-Data Unit of the .fits file the visibilitiy data is loaded from.
         """
-        visibilities_1d = array_util.numpy_array_2d_from_fits(
+        visibilities_1d = array_2d_util.numpy_array_2d_from_fits(
             file_path=file_path, hdu=hdu
         )
         return cls.manual_slim(visibilities=visibilities_1d)
